@@ -4,11 +4,11 @@
 # See top-level LICENSE file for more information
 
 import datetime
+import itertools
 import json
 import random
 import re
 from typing import Any, Dict, List, Iterable, Optional, Union
-import uuid
 
 import attr
 import dateutil
@@ -21,6 +21,7 @@ from swh.model.model import (
     Content,
     SkippedContent,
     OriginVisit,
+    OriginVisitStatus,
     Snapshot,
     Origin,
 )
@@ -542,7 +543,7 @@ class CassandraStorage:
         snapshots = [snp for snp in snapshots if snp.id in missing]
 
         for snapshot in snapshots:
-            self.journal_writer.snapshot_add(snapshot)
+            self.journal_writer.snapshot_add([snapshot])
 
             # Add branches
             for (branch_name, branch) in snapshot.branches.items():
@@ -576,11 +577,11 @@ class CassandraStorage:
 
     def snapshot_get_by_origin_visit(self, origin, visit):
         try:
-            visit = self._cql_runner.origin_visit_get_one(origin, visit)
+            visit = self.origin_visit_get_by(origin, visit)
         except IndexError:
             return None
 
-        return self.snapshot_get(visit.snapshot)
+        return self.snapshot_get(visit["snapshot"])
 
     def snapshot_get_latest(self, origin, allowed_statuses=None):
         visit = self.origin_visit_get_latest(
@@ -786,7 +787,7 @@ class CassandraStorage:
         if known_origin:
             origin_url = known_origin["url"]
         else:
-            self.journal_writer.origin_add_one(origin)
+            self.journal_writer.origin_add([origin])
 
             self._cql_runner.origin_add_one(origin)
             origin_url = origin.url
@@ -806,24 +807,40 @@ class CassandraStorage:
             raise StorageArgumentException("Unknown origin %s", origin_url)
 
         visit_id = self._cql_runner.origin_generate_unique_visit_id(origin_url)
-
+        visit_state = "ongoing"
         with convert_validation_exceptions():
             visit = OriginVisit.from_dict(
                 {
                     "origin": origin_url,
                     "date": date,
                     "type": type,
-                    "status": "ongoing",
+                    "status": visit_state,
                     "snapshot": None,
                     "metadata": None,
                     "visit": visit_id,
                 }
             )
 
-        self.journal_writer.origin_visit_add(visit)
+        self.journal_writer.origin_visit_add([visit])
         self._cql_runner.origin_visit_add_one(visit)
+
+        with convert_validation_exceptions():
+            visit_status = OriginVisitStatus(
+                origin=origin_url,
+                visit=visit_id,
+                date=date,
+                status=visit_state,
+                snapshot=None,
+                metadata=None,
+            )
+        self._origin_visit_status_add(visit_status)
+
         return visit
 
+    def _origin_visit_status_add(self, visit_status: OriginVisitStatus) -> None:
+        """Add an origin visit status"""
+        self._cql_runner.origin_visit_status_add_one(visit_status)
+
     def origin_visit_update(
         self,
         origin: str,
@@ -836,24 +853,77 @@ class CassandraStorage:
         origin_url = origin  # TODO: rename the argument
 
         # Get the existing data of the visit
-        row = self._cql_runner.origin_visit_get_one(origin_url, visit_id)
-        if not row:
+        visit_ = self.origin_visit_get_by(origin_url, visit_id)
+        if not visit_:
             raise StorageArgumentException("This origin visit does not exist.")
         with convert_validation_exceptions():
-            visit = OriginVisit.from_dict(self._format_origin_visit_row(row))
+            visit = OriginVisit.from_dict(visit_)
 
         updates: Dict[str, Any] = {"status": status}
-        if metadata:
+        if metadata and metadata != visit.metadata:
             updates["metadata"] = metadata
-        if snapshot:
+        if snapshot and snapshot != visit.snapshot:
             updates["snapshot"] = snapshot
 
         with convert_validation_exceptions():
             visit = attr.evolve(visit, **updates)
 
-        self.journal_writer.origin_visit_update(visit)
+        self.journal_writer.origin_visit_update([visit])
 
-        self._cql_runner.origin_visit_update(origin_url, visit_id, updates)
+        last_visit_update = self._origin_visit_get_updated(visit.origin, visit.visit)
+        assert last_visit_update is not None
+
+        with convert_validation_exceptions():
+            visit_status = OriginVisitStatus(
+                origin=origin_url,
+                visit=visit_id,
+                date=date or now(),
+                status=status,
+                snapshot=snapshot or last_visit_update["snapshot"],
+                metadata=metadata or last_visit_update["metadata"],
+            )
+        self._origin_visit_status_add(visit_status)
+
+    def _origin_visit_merge(
+        self, visit: Dict[str, Any], visit_status: Dict[str, Any]
+    ) -> Dict[str, Any]:
+        """Merge origin_visit and visit_status together.
+
+        """
+        return OriginVisit.from_dict(
+            {
+                # default to the values in visit
+                **visit,
+                # override with the last update
+                **visit_status,
+                # visit['origin'] is the URL (via a join), while
+                # visit_status['origin'] is only an id.
+                "origin": visit["origin"],
+                # but keep the date of the creation of the origin visit
+                "date": visit["date"],
+            }
+        ).to_dict()
+
+    def _origin_visit_apply_last_status(self, visit: Dict[str, Any]) -> Dict[str, Any]:
+        """Retrieve the latest visit status information for the origin visit.
+        Then merge it with the visit and return it.
+
+        """
+        visit_status = self._cql_runner.origin_visit_status_get_latest(
+            visit["origin"], visit["visit"]
+        )
+        assert visit_status is not None
+        return self._origin_visit_merge(visit, visit_status)
+
+    def _origin_visit_get_updated(self, origin: str, visit_id: int) -> Dict[str, Any]:
+        """Retrieve origin visit and latest origin visit status and merge them
+        into an origin visit.
+
+        """
+        row_visit = self._cql_runner.origin_visit_get_one(origin, visit_id)
+        assert row_visit is not None
+        visit = self._format_origin_visit_row(row_visit)
+        return self._origin_visit_apply_last_status(visit)
 
     def origin_visit_upsert(self, visits: Iterable[OriginVisit]) -> None:
         for visit in visits:
@@ -862,7 +932,18 @@ class CassandraStorage:
 
         self.journal_writer.origin_visit_upsert(visits)
         for visit in visits:
+            assert visit.visit is not None
             self._cql_runner.origin_visit_upsert(visit)
+            with convert_validation_exceptions():
+                visit_status = OriginVisitStatus(
+                    origin=visit.origin,
+                    visit=visit.visit,
+                    date=now(),
+                    status=visit.status,
+                    snapshot=visit.snapshot,
+                    metadata=visit.metadata,
+                )
+            self._origin_visit_status_add(visit_status)
 
     @staticmethod
     def _format_origin_visit_row(visit):
@@ -877,8 +958,9 @@ class CassandraStorage:
         self, origin: str, last_visit: Optional[int] = None, limit: Optional[int] = None
     ) -> Iterable[Dict[str, Any]]:
         rows = self._cql_runner.origin_visit_get(origin, last_visit, limit)
-
-        yield from map(self._format_origin_visit_row, rows)
+        for row in rows:
+            visit = self._format_origin_visit_row(row)
+            yield self._origin_visit_apply_last_status(visit)
 
     def origin_visit_find_by_date(
         self, origin: str, visit_date: datetime.datetime
@@ -886,21 +968,23 @@ class CassandraStorage:
         # Iterator over all the visits of the origin
         # This should be ok for now, as there aren't too many visits
         # per origin.
-        visits = list(self._cql_runner.origin_visit_get_all(origin))
+        rows = list(self._cql_runner.origin_visit_get_all(origin))
 
         def key(visit):
             dt = visit.date.replace(tzinfo=datetime.timezone.utc) - visit_date
             return (abs(dt), -visit.visit)
 
-        if visits:
-            visit = min(visits, key=key)
-            return visit._asdict()
+        if rows:
+            row = min(rows, key=key)
+            visit = self._format_origin_visit_row(row)
+            return self._origin_visit_apply_last_status(visit)
         return None
 
     def origin_visit_get_by(self, origin: str, visit: int) -> Optional[Dict[str, Any]]:
-        visit = self._cql_runner.origin_visit_get_one(origin, visit)
-        if visit:
-            return self._format_origin_visit_row(visit)
+        row = self._cql_runner.origin_visit_get_one(origin, visit)
+        if row:
+            visit_ = self._format_origin_visit_row(row)
+            return self._origin_visit_apply_last_status(visit_)
         return None
 
     def origin_visit_get_latest(
@@ -909,12 +993,27 @@ class CassandraStorage:
         allowed_statuses: Optional[List[str]] = None,
         require_snapshot: bool = False,
     ) -> Optional[Dict[str, Any]]:
-        visit = self._cql_runner.origin_visit_get_latest(
-            origin, allowed_statuses=allowed_statuses, require_snapshot=require_snapshot
-        )
-        if visit:
-            return self._format_origin_visit_row(visit)
-        return None
+        # TODO: Do not fetch all visits
+        rows = self._cql_runner.origin_visit_get_all(origin)
+        latest_visit = None
+        for row in rows:
+            visit = self._format_origin_visit_row(row)
+            updated_visit = self._origin_visit_apply_last_status(visit)
+            if allowed_statuses and updated_visit["status"] not in allowed_statuses:
+                continue
+            if require_snapshot and updated_visit["snapshot"] is None:
+                continue
+
+            # updated_visit is a candidate
+            if latest_visit is not None:
+                if updated_visit["date"] < latest_visit["date"]:
+                    continue
+                if updated_visit["visit"] < latest_visit["visit"]:
+                    continue
+
+            latest_visit = updated_visit
+
+        return latest_visit
 
     def origin_visit_get_random(self, type: str) -> Optional[Dict[str, Any]]:
         back_in_the_day = now() - datetime.timedelta(weeks=12)  # 3 months back
@@ -926,39 +1025,12 @@ class CassandraStorage:
         rows = self._cql_runner.origin_visit_iter(start_token)
         for row in rows:
             visit = self._format_origin_visit_row(row)
-            if visit["date"] > back_in_the_day and visit["status"] == "full":
-                return visit
-        else:
-            return None
-
-    def tool_add(self, tools):
-        inserted = []
-        for tool in tools:
-            tool = tool.copy()
-            tool_json = tool.copy()
-            tool_json["configuration"] = json.dumps(
-                tool["configuration"], sort_keys=True
-            ).encode()
-            id_ = self._cql_runner.tool_get_one_uuid(**tool_json)
-            if not id_:
-                id_ = uuid.uuid1()
-                tool_json["id"] = id_
-                self._cql_runner.tool_by_uuid_add_one(tool_json)
-                self._cql_runner.tool_add_one(tool_json)
-            tool["id"] = id_
-            inserted.append(tool)
-        return inserted
-
-    def tool_get(self, tool):
-        id_ = self._cql_runner.tool_get_one_uuid(
-            tool["name"],
-            tool["version"],
-            json.dumps(tool["configuration"], sort_keys=True).encode(),
-        )
-        if id_:
-            tool = tool.copy()
-            tool["id"] = id_
-            return tool
+            visit_status = self._origin_visit_apply_last_status(visit)
+            if (
+                visit_status["date"] > back_in_the_day
+                and visit_status["status"] == "full"
+            ):
+                return visit_status
         else:
             return None
 
@@ -981,27 +1053,112 @@ class CassandraStorage:
     def refresh_stat_counters(self):
         pass
 
-    def origin_metadata_add(self, origin_url, ts, provider, tool, metadata):
-        # TODO
-        raise NotImplementedError("not yet supported for Cassandra")
+    def origin_metadata_add(
+        self,
+        origin_url: str,
+        discovery_date: datetime.datetime,
+        authority: Dict[str, Any],
+        fetcher: Dict[str, Any],
+        format: str,
+        metadata: bytes,
+    ) -> None:
+        if not isinstance(origin_url, str):
+            raise StorageArgumentException(
+                "origin_id must be str, not %r" % (origin_url,)
+            )
+        if not self._cql_runner.metadata_authority_get(**authority):
+            raise StorageArgumentException(f"Unknown authority {authority}")
+        if not self._cql_runner.metadata_fetcher_get(**fetcher):
+            raise StorageArgumentException(f"Unknown fetcher {fetcher}")
 
-    def origin_metadata_get_by(self, origin_url, provider_type=None):
-        # TODO
-        raise NotImplementedError("not yet supported for Cassandra")
+        try:
+            self._cql_runner.origin_metadata_add(
+                origin_url,
+                authority["type"],
+                authority["url"],
+                discovery_date,
+                fetcher["name"],
+                fetcher["version"],
+                format,
+                metadata,
+            )
+        except TypeError as e:
+            raise StorageArgumentException(*e.args)
 
-    def metadata_provider_add(
-        self, provider_name, provider_type, provider_url, metadata
-    ):
-        # TODO
-        raise NotImplementedError("not yet supported for Cassandra")
+    def origin_metadata_get(
+        self,
+        origin_url: str,
+        authority: Dict[str, str],
+        after: Optional[datetime.datetime] = None,
+        limit: Optional[int] = None,
+    ) -> List[Dict[str, Any]]:
+        if not isinstance(origin_url, str):
+            raise TypeError("origin_url must be str, not %r" % (origin_url,))
 
-    def metadata_provider_get(self, provider_id):
-        # TODO
-        raise NotImplementedError("not yet supported for Cassandra")
+        if after is None:
+            entries = self._cql_runner.origin_metadata_get(
+                origin_url, authority["type"], authority["url"]
+            )
+        else:
+            entries = self._cql_runner.origin_metadata_get_after(
+                origin_url, authority["type"], authority["url"], after
+            )
 
-    def metadata_provider_get_by(self, provider):
-        # TODO
-        raise NotImplementedError("not yet supported for Cassandra")
+        if limit:
+            entries = itertools.islice(entries, 0, limit)
+
+        results = []
+        for entry in entries:
+            discovery_date = entry.discovery_date.replace(tzinfo=datetime.timezone.utc)
+            results.append(
+                {
+                    "origin_url": entry.origin,
+                    "authority": {
+                        "type": entry.authority_type,
+                        "url": entry.authority_url,
+                    },
+                    "fetcher": {
+                        "name": entry.fetcher_name,
+                        "version": entry.fetcher_version,
+                    },
+                    "discovery_date": discovery_date,
+                    "format": entry.format,
+                    "metadata": entry.metadata,
+                }
+            )
+        return results
+
+    def metadata_fetcher_add(
+        self, name: str, version: str, metadata: Dict[str, Any]
+    ) -> None:
+        self._cql_runner.metadata_fetcher_add(name, version, json.dumps(metadata))
+
+    def metadata_fetcher_get(self, name: str, version: str) -> Optional[Dict[str, Any]]:
+        fetcher = self._cql_runner.metadata_fetcher_get(name, version)
+        if fetcher:
+            return {
+                "name": fetcher.name,
+                "version": fetcher.version,
+                "metadata": json.loads(fetcher.metadata),
+            }
+        else:
+            return None
+
+    def metadata_authority_add(
+        self, type: str, url: str, metadata: Dict[str, Any]
+    ) -> None:
+        self._cql_runner.metadata_authority_add(url, type, json.dumps(metadata))
+
+    def metadata_authority_get(self, type: str, url: str) -> Optional[Dict[str, Any]]:
+        authority = self._cql_runner.metadata_authority_get(type, url)
+        if authority:
+            return {
+                "type": authority.type,
+                "url": authority.url,
+                "metadata": json.loads(authority.metadata),
+            }
+        else:
+            return None
 
     def clear_buffers(self, object_types: Optional[Iterable[str]] = None) -> None:
         """Do nothing
