@@ -3,13 +3,15 @@
 # License: GNU General Public License version 3, or any later version
 # See top-level LICENSE file for more information
 
+import datetime
 import random
 import select
+from typing import Any, Dict, Optional, Tuple
 
 from swh.core.db import BaseDb
 from swh.core.db.db_utils import stored_procedure, jsonize
 from swh.core.db.db_utils import execute_values_generator
-from swh.model.model import OriginVisit, SHA1_SIZE
+from swh.model.model import OriginVisit, OriginVisitStatus, SHA1_SIZE
 
 
 class Db(BaseDb):
@@ -440,6 +442,37 @@ class Db(BaseDb):
         )
         return cur.fetchone()[0]
 
+    origin_visit_status_cols = [
+        "origin",
+        "visit",
+        "date",
+        "status",
+        "snapshot",
+        "metadata",
+    ]
+
+    def origin_visit_status_add(
+        self, visit_status: OriginVisitStatus, cur=None
+    ) -> None:
+        """Add new origin visit status
+
+        """
+        assert self.origin_visit_status_cols[0] == "origin"
+        assert self.origin_visit_status_cols[-1] == "metadata"
+        cols = self.origin_visit_status_cols[1:-1]
+        cur = self._cursor(cur)
+        cur.execute(
+            f"WITH origin_id as (select id from origin where url=%s) "
+            f"INSERT INTO origin_visit_status "
+            f"(origin, {', '.join(cols)}, metadata) "
+            f"VALUES ((select id from origin_id), "
+            f"{', '.join(['%s']*len(cols))}, %s) "
+            f"ON CONFLICT (origin, visit, date) do nothing",
+            [visit_status.origin]
+            + [getattr(visit_status, key) for key in cols]
+            + [jsonize(visit_status.metadata)],
+        )
+
     def origin_visit_update(self, origin_id, visit_id, updates, cur=None):
         """Update origin_visit's status."""
         cur = self._cursor(cur)
@@ -513,6 +546,33 @@ class Db(BaseDb):
         "snapshot",
     ]
 
+    def _make_origin_visit_status(self, row: Tuple[Any]) -> Optional[Dict[str, Any]]:
+        """Make an origin_visit_status dict out of a row
+
+        """
+        if not row:
+            return None
+        return dict(zip(self.origin_visit_status_cols, row))
+
+    def origin_visit_status_get_latest(
+        self, origin: str, visit: int, cur=None
+    ) -> Optional[Dict[str, Any]]:
+        """Given an origin visit id, return its latest origin_visit_status
+
+        """
+        cols = self.origin_visit_status_cols
+        cur = self._cursor(cur)
+        cur.execute(
+            f"SELECT {', '.join(cols)} "
+            f"FROM origin_visit_status ovs "
+            f"INNER JOIN origin o on o.id=ovs.origin "
+            f"WHERE o.url=%s AND ovs.visit=%s"
+            f"ORDER BY ovs.date DESC LIMIT 1",
+            (origin, visit),
+        )
+        row = cur.fetchone()
+        return self._make_origin_visit_status(row)
+
     def origin_visit_get_all(self, origin_id, last_visit=None, limit=None, cur=None):
         """Retrieve all visits for origin with id origin_id.
 
@@ -520,7 +580,7 @@ class Db(BaseDb):
             origin_id: The occurrence's origin
 
         Yields:
-            The occurrence's history visits
+            The visits for that origin
 
         """
         cur = self._cursor(cur)
@@ -533,12 +593,12 @@ class Db(BaseDb):
             args = (origin_id, limit)
 
         query = """\
-        SELECT %s
-        FROM origin_visit
-        INNER JOIN origin ON origin.id = origin_visit.origin
-        WHERE origin.url=%%s %s
-        order by visit asc
-        limit %%s""" % (
+         SELECT %s
+         FROM origin_visit
+         INNER JOIN origin ON origin.id = origin_visit.origin
+         WHERE origin.url=%%s %s
+         order by visit asc
+         limit %%s""" % (
             ", ".join(self.origin_visit_select_cols),
             extra_condition,
         )
@@ -580,9 +640,11 @@ class Db(BaseDb):
         cur.execute(
             "SELECT * FROM swh_visit_find_by_date(%s, %s)", (origin, visit_date)
         )
-        r = cur.fetchall()
-        if r:
-            return r[0]
+        rows = cur.fetchall()
+        if rows:
+            visit = dict(zip(self.origin_visit_get_cols, rows[0]))
+            visit["origin"] = origin
+            return visit
 
     def origin_visit_exists(self, origin_id, visit_id, cur=None):
         """Check whether an origin visit with the given ids exists"""
@@ -997,135 +1059,150 @@ class Db(BaseDb):
     def release_get_random(self, cur=None):
         return self._get_random_row_from_table("release", ["id"], "id", cur)
 
-    def origin_metadata_add(self, origin, ts, provider, tool, metadata, cur=None):
+    origin_metadata_get_cols = [
+        "origin.url",
+        "discovery_date",
+        "metadata_authority.type",
+        "metadata_authority.url",
+        "metadata_fetcher.name",
+        "metadata_fetcher.version",
+        "format",
+        "metadata",
+    ]
+
+    def origin_metadata_add(
+        self,
+        origin: str,
+        discovery_date: datetime.datetime,
+        authority: int,
+        fetcher: int,
+        format: str,
+        metadata: bytes,
+        cur=None,
+    ) -> None:
         """ Add an origin_metadata for the origin at ts with provider, tool and
         metadata.
 
         Args:
-            origin (int): the origin's id for which the metadata is added
-            ts (datetime): time when the metadata was found
-            provider (int): the metadata provider identifier
-            tool (int): the tool's identifier used to extract metadata
-            metadata (jsonb): the metadata retrieved at the time and location
-
-        Returns:
-            id (int): the origin_metadata unique id
-
+            origin: the origin's id for which the metadata is added
+            discovery_date: time when the metadata was found
+            authority: the metadata provider identifier
+            fetcher: the tool's identifier used to extract metadata
+            format: the format of the metadata
+            metadata: the metadata retrieved at the time and location
         """
         cur = self._cursor(cur)
         insert = """INSERT INTO origin_metadata (origin_id, discovery_date,
-                    provider_id, tool_id, metadata)
-                    SELECT id, %s, %s, %s, %s FROM origin WHERE url = %s"""
-        cur.execute(insert, (ts, provider, tool, jsonize(metadata), origin))
-
-    origin_metadata_get_cols = [
-        "origin_url",
-        "discovery_date",
-        "tool_id",
-        "metadata",
-        "provider_id",
-        "provider_name",
-        "provider_type",
-        "provider_url",
-    ]
-
-    def origin_metadata_get_by(self, origin_url, provider_type=None, cur=None):
-        """Retrieve all origin_metadata entries for one origin_url
-
-        """
-        cur = self._cursor(cur)
-        if not provider_type:
-            query = """SELECT %s
-                       FROM swh_origin_metadata_get_by_origin(
-                            %%s)""" % (
-                ",".join(self.origin_metadata_get_cols)
-            )
-
-            cur.execute(query, (origin_url,))
-
-        else:
-            query = """SELECT %s
-                       FROM swh_origin_metadata_get_by_provider_type(
-                            %%s, %%s)""" % (
-                ",".join(self.origin_metadata_get_cols)
-            )
-
-            cur.execute(query, (origin_url, provider_type))
-
-        yield from cur
-
-    tool_cols = ["id", "name", "version", "configuration"]
-
-    @stored_procedure("swh_mktemp_tool")
-    def mktemp_tool(self, cur=None):
-        pass
-
-    def tool_add_from_temp(self, cur=None):
-        cur = self._cursor(cur)
-        cur.execute("SELECT %s from swh_tool_add()" % (",".join(self.tool_cols),))
-        yield from cur
-
-    def tool_get(self, name, version, configuration, cur=None):
-        cur = self._cursor(cur)
+                    authority_id, fetcher_id, format, metadata)
+                    SELECT id, %s, %s, %s, %s, %s FROM origin WHERE url = %s"""
         cur.execute(
-            """select %s
-                       from tool
-                       where name=%%s and
-                             version=%%s and
-                             configuration=%%s"""
-            % (",".join(self.tool_cols)),
-            (name, version, configuration),
+            insert, (discovery_date, authority, fetcher, format, metadata, origin),
         )
 
-        return cur.fetchone()
-
-    metadata_provider_cols = [
-        "id",
-        "provider_name",
-        "provider_type",
-        "provider_url",
-        "metadata",
-    ]
-
-    def metadata_provider_add(
-        self, provider_name, provider_type, provider_url, metadata, cur=None
+    def origin_metadata_get(
+        self,
+        origin_url: str,
+        authority: int,
+        after: Optional[datetime.datetime],
+        limit: Optional[int],
+        cur=None,
     ):
-        """Insert a new provider and return the new identifier."""
         cur = self._cursor(cur)
-        insert = """INSERT INTO metadata_provider (provider_name, provider_type,
-                    provider_url, metadata) values (%s, %s, %s, %s)
-                    RETURNING id"""
+        assert self.origin_metadata_get_cols[-1] == "metadata"
+        query_parts = [
+            f"SELECT {', '.join(self.origin_metadata_get_cols[0:-1])}, "
+            f"  origin_metadata.metadata AS metadata "
+            f"FROM origin_metadata "
+            f"INNER JOIN metadata_authority "
+            f"  ON (metadata_authority.id=authority_id) "
+            f"INNER JOIN metadata_fetcher ON (metadata_fetcher.id=fetcher_id) "
+            f"INNER JOIN origin ON (origin.id=origin_metadata.origin_id) "
+            f"WHERE origin.url=%s AND authority_id=%s"
+        ]
+        args = [origin_url, authority]
 
-        cur.execute(
-            insert, (provider_name, provider_type, provider_url, jsonize(metadata))
-        )
-        return cur.fetchone()[0]
+        if after:
+            query_parts.append("AND discovery_date >= %s")
+            args.append(after)
 
-    def metadata_provider_get(self, provider_id, cur=None):
+        query_parts.append("ORDER BY discovery_date")
+
+        if limit:
+            query_parts.append("LIMIT %s")
+            args.append(limit)
+
+        cur.execute(" ".join(query_parts), args)
+        yield from cur
+
+    metadata_fetcher_cols = ["name", "version", "metadata"]
+
+    def metadata_fetcher_add(
+        self, name: str, version: str, metadata: bytes, cur=None
+    ) -> None:
         cur = self._cursor(cur)
         cur.execute(
-            """select %s
-                       from metadata_provider
-                       where id=%%s """
-            % (",".join(self.metadata_provider_cols)),
-            (provider_id,),
+            "INSERT INTO metadata_fetcher (name, version, metadata) "
+            "VALUES (%s, %s, %s) ON CONFLICT DO NOTHING",
+            (name, version, jsonize(metadata)),
         )
 
+    def metadata_fetcher_get(self, name: str, version: str, cur=None):
+        cur = self._cursor(cur)
+        cur.execute(
+            f"SELECT {', '.join(self.metadata_fetcher_cols)} "
+            f"FROM metadata_fetcher "
+            f"WHERE name=%s AND version=%s",
+            (name, version),
+        )
         return cur.fetchone()
 
-    def metadata_provider_get_by(self, provider_name, provider_url, cur=None):
+    def metadata_fetcher_get_id(
+        self, name: str, version: str, cur=None
+    ) -> Optional[int]:
         cur = self._cursor(cur)
         cur.execute(
-            """select %s
-                       from metadata_provider
-                       where provider_name=%%s and
-                             provider_url=%%s"""
-            % (",".join(self.metadata_provider_cols)),
-            (provider_name, provider_url),
+            "SELECT id FROM metadata_fetcher WHERE name=%s AND version=%s",
+            (name, version),
+        )
+        row = cur.fetchone()
+        if row:
+            return row[0]
+        else:
+            return None
+
+    metadata_authority_cols = ["type", "url", "metadata"]
+
+    def metadata_authority_add(
+        self, type: str, url: str, metadata: bytes, cur=None
+    ) -> None:
+        cur = self._cursor(cur)
+        cur.execute(
+            "INSERT INTO metadata_authority (type, url, metadata) "
+            "VALUES (%s, %s, %s) ON CONFLICT DO NOTHING",
+            (type, url, jsonize(metadata)),
         )
 
+    def metadata_authority_get(self, type: str, url: str, cur=None):
+        cur = self._cursor(cur)
+        cur.execute(
+            f"SELECT {', '.join(self.metadata_authority_cols)} "
+            f"FROM metadata_authority "
+            f"WHERE type=%s AND url=%s",
+            (type, url),
+        )
         return cur.fetchone()
 
+    def metadata_authority_get_id(self, type: str, url: str, cur=None) -> Optional[int]:
+        cur = self._cursor(cur)
+        cur.execute(
+            "SELECT id FROM metadata_authority WHERE type=%s AND url=%s", (type, url)
+        )
+        row = cur.fetchone()
+        if row:
+            return row[0]
+        else:
+            return None
+
     def _get_random_row_from_table(self, table_name, cols, id_col, cur=None):
         random_sha1 = bytes(random.randint(0, 255) for _ in range(SHA1_SIZE))
         cur = self._cursor(cur)
