@@ -14,7 +14,20 @@ import random
 
 from collections import defaultdict
 from datetime import timedelta
-from typing import Any, Dict, Iterable, List, Optional, Union
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Generic,
+    Hashable,
+    Iterable,
+    Iterator,
+    List,
+    Optional,
+    Tuple,
+    TypeVar,
+    Union,
+)
 
 import attr
 
@@ -27,6 +40,7 @@ from swh.model.model import (
     Release,
     Snapshot,
     OriginVisit,
+    OriginVisitStatus,
     Origin,
     SHA1_SIZE,
 )
@@ -45,6 +59,50 @@ from .writer import JournalWriter
 BULK_BLOCK_CONTENT_LEN_MAX = 10000
 
 
+SortedListItem = TypeVar("SortedListItem")
+SortedListKey = TypeVar("SortedListKey")
+
+
+class SortedList(collections.UserList, Generic[SortedListKey, SortedListItem]):
+    data: List[Tuple[SortedListKey, SortedListItem]]
+
+    # https://github.com/python/mypy/issues/708
+    # key: Callable[[SortedListItem], SortedListKey]
+
+    def __init__(
+        self,
+        data: List[SortedListItem] = None,
+        key: Optional[Callable[[SortedListItem], SortedListKey]] = None,
+    ):
+        if key is None:
+
+            def key(item):
+                return item
+
+        assert key is not None  # for mypy
+        super().__init__(sorted((key(x), x) for x in data or []))
+
+        self.key: Callable[[SortedListItem], SortedListKey] = key
+
+    def add(self, item: SortedListItem):
+        k = self.key(item)
+        bisect.insort(self.data, (k, item))
+
+    def __iter__(self) -> Iterator[SortedListItem]:
+        for (k, item) in self.data:
+            yield item
+
+    def iter_from(self, start_key: SortedListKey) -> Iterator[SortedListItem]:
+        """Returns an iterator over all the elements whose key is greater
+        or equal to `start_key`.
+        (This is an efficient equivalent to:
+        `(x for x in L if key(x) >= start_key)`)
+        """
+        from_index = bisect.bisect_left(self.data, (start_key,))
+        for (k, item) in itertools.islice(self.data, from_index, None):
+            yield item
+
+
 class InMemoryStorage:
     def __init__(self, journal_writer=None):
 
@@ -64,14 +122,20 @@ class InMemoryStorage:
         self._origins_by_id = []
         self._origins_by_sha1 = {}
         self._origin_visits = {}
+        self._origin_visit_statuses: Dict[Tuple[str, int], List[OriginVisitStatus]] = {}
         self._persons = []
-        self._origin_metadata = defaultdict(list)
-        self._tools = {}
-        self._metadata_providers = {}
-        self._objects = defaultdict(list)
 
-        # ideally we would want a skip list for both fast inserts and searches
-        self._sorted_sha1s = []
+        # {origin_url: {authority: [metadata]}}
+        self._origin_metadata: Dict[
+            str, Dict[Hashable, SortedList[datetime.datetime, Dict[str, Any]]]
+        ] = defaultdict(
+            lambda: defaultdict(lambda: SortedList(key=lambda x: x["discovery_date"]))
+        )  # noqa
+
+        self._metadata_fetchers: Dict[Hashable, Dict[str, Any]] = {}
+        self._metadata_authorities: Dict[Hashable, Dict[str, Any]] = {}
+        self._objects = defaultdict(list)
+        self._sorted_sha1s = SortedList[bytes, bytes]()
 
         self.objstorage = ObjStorage({"cls": "memory", "args": {}})
 
@@ -82,7 +146,6 @@ class InMemoryStorage:
         self.journal_writer.content_add(contents)
 
         content_add = 0
-        content_add_bytes = 0
         if with_data:
             summary = self.objstorage.content_add(
                 c for c in contents if c.status != "absent"
@@ -111,7 +174,7 @@ class InMemoryStorage:
                 self._content_indexes[algorithm][hash_].add(key)
             self._objects[content.sha1_git].append(("content", content.sha1))
             self._contents[key] = content
-            bisect.insort(self._sorted_sha1s, content.sha1)
+            self._sorted_sha1s.add(content.sha1)
             self._contents[key] = attr.evolve(self._contents[key], data=None)
             content_add += 1
 
@@ -163,11 +226,9 @@ class InMemoryStorage:
     def content_get_range(self, start, end, limit=1000):
         if limit is None:
             raise StorageArgumentException("limit should not be None")
-        from_index = bisect.bisect_left(self._sorted_sha1s, start)
-        sha1s = itertools.islice(self._sorted_sha1s, from_index, None)
         sha1s = (
             (sha1, content_key)
-            for sha1 in sha1s
+            for sha1 in self._sorted_sha1s.iter_from(start)
             for content_key in self._content_indexes["sha1"][sha1]
         )
         matched = []
@@ -474,7 +535,7 @@ class InMemoryStorage:
         count = 0
         snapshots = (snap for snap in snapshots if snap.id not in self._snapshots)
         for snapshot in snapshots:
-            self.journal_writer.snapshot_add(snapshot)
+            self.journal_writer.snapshot_add([snapshot])
             sorted_branch_names = sorted(snapshot.branches)
             self._snapshots[snapshot.id] = (snapshot, sorted_branch_names)
             self._objects[snapshot.id].append(("snapshot", snapshot.id))
@@ -499,7 +560,9 @@ class InMemoryStorage:
             self._origin_visits[origin_url]
         ):
             return None
-        snapshot_id = self._origin_visits[origin_url][visit - 1].snapshot
+
+        visit = self._origin_visit_get_updated(origin_url, visit)
+        snapshot_id = visit.snapshot
         if snapshot_id:
             return self.snapshot_get(snapshot_id)
         else:
@@ -666,19 +729,20 @@ class InMemoryStorage:
         else:
             origins = [orig for orig in origins if url_pattern in orig["url"]]
         if with_visit:
-            origins = [
-                orig
-                for orig in origins
-                if len(self._origin_visits[orig["url"]]) > 0
-                and set(
-                    ov.snapshot
+            filtered_origins = []
+            for orig in origins:
+                visits = (
+                    self._origin_visit_get_updated(ov.origin, ov.visit)
                     for ov in self._origin_visits[orig["url"]]
-                    if ov.snapshot
                 )
-                & set(self._snapshots)
-            ]
+                for ov in visits:
+                    if ov.snapshot and ov.snapshot in self._snapshots:
+                        filtered_origins.append(orig)
+                        break
+        else:
+            filtered_origins = origins
 
-        return origins[offset : offset + limit]
+        return filtered_origins[offset : offset + limit]
 
     def origin_count(self, url_pattern, regexp=False, with_visit=False):
         return len(
@@ -698,7 +762,7 @@ class InMemoryStorage:
 
     def origin_add_one(self, origin: Origin) -> str:
         if origin.url not in self._origins:
-            self.journal_writer.origin_add_one(origin)
+            self.journal_writer.origin_add([origin])
             # generate an origin_id because it is needed by origin_get_range.
             # TODO: remove this when we remove origin_get_range
             origin_id = len(self._origins) + 1
@@ -730,21 +794,35 @@ class InMemoryStorage:
             # visit ids are in the range [1, +inf[
             visit_id = len(self._origin_visits[origin_url]) + 1
             status = "ongoing"
-            visit = OriginVisit(
-                origin=origin_url,
-                date=date,
-                type=type,
-                status=status,
-                snapshot=None,
-                metadata=None,
-                visit=visit_id,
-            )
+            with convert_validation_exceptions():
+                visit = OriginVisit(
+                    origin=origin_url,
+                    date=date,
+                    type=type,
+                    # TODO: Remove when we remove those fields from the model
+                    status=status,
+                    snapshot=None,
+                    metadata=None,
+                    visit=visit_id,
+                )
             self._origin_visits[origin_url].append(visit)
-            visit = visit
+            assert visit.visit is not None
+            visit_key = (origin_url, visit.visit)
 
-            self._objects[(origin_url, visit.visit)].append(("origin_visit", None))
+            with convert_validation_exceptions():
+                visit_update = OriginVisitStatus(
+                    origin=origin_url,
+                    visit=visit_id,
+                    date=date,
+                    status=status,
+                    snapshot=None,
+                    metadata=None,
+                )
+            self._origin_visit_statuses[visit_key] = [visit_update]
 
-            self.journal_writer.origin_visit_add(visit)
+            self._objects[visit_key].append(("origin_visit", None))
+
+            self.journal_writer.origin_visit_add([visit])
 
         # return last visit
         return visit
@@ -767,16 +845,28 @@ class InMemoryStorage:
         except IndexError:
             raise StorageArgumentException("Unknown visit_id for this origin") from None
 
-        updates: Dict[str, Any] = {"status": status}
-        if metadata:
-            updates["metadata"] = metadata
-        if snapshot:
-            updates["snapshot"] = snapshot
+        # Retrieve the previous visit status
+        assert visit.visit is not None
+        visit_key = (origin_url, visit.visit)
+
+        last_visit_update = max(
+            self._origin_visit_statuses[visit_key], key=lambda v: v.date
+        )
 
         with convert_validation_exceptions():
-            visit = attr.evolve(visit, **updates)
+            visit_update = OriginVisitStatus(
+                origin=origin_url,
+                visit=visit_id,
+                date=date or now(),
+                status=status,
+                snapshot=snapshot or last_visit_update.snapshot,
+                metadata=metadata or last_visit_update.metadata,
+            )
+        self._origin_visit_statuses[visit_key].append(visit_update)
 
-        self.journal_writer.origin_visit_update(visit)
+        self.journal_writer.origin_visit_update(
+            [self._origin_visit_get_updated(origin_url, visit_id)]
+        )
 
         self._origin_visits[origin_url][visit_id - 1] = visit
 
@@ -787,28 +877,62 @@ class InMemoryStorage:
 
         self.journal_writer.origin_visit_upsert(visits)
 
+        date = now()
+
         for visit in visits:
-            visit_id = visit.visit
+            assert visit.visit is not None
             origin_url = visit.origin
+            origin = self.origin_get({"url": origin_url})
 
-            with convert_validation_exceptions():
-                visit = attr.evolve(visit, origin=origin_url)
+            if not origin:  # Cannot add a visit without an origin
+                raise StorageArgumentException("Unknown origin %s", origin_url)
 
-            self._objects[(origin_url, visit_id)].append(("origin_visit", None))
+            if origin_url in self._origins:
+                origin = self._origins[origin_url]
+                # visit ids are in the range [1, +inf[
+                assert visit.visit is not None
+                visit_key = (origin_url, visit.visit)
 
-            if visit_id:
-                while len(self._origin_visits[origin_url]) <= visit_id:
+                with convert_validation_exceptions():
+                    visit_update = OriginVisitStatus(
+                        origin=origin_url,
+                        visit=visit.visit,
+                        date=date,
+                        status=visit.status,
+                        snapshot=visit.snapshot,
+                        metadata=visit.metadata,
+                    )
+
+                self._origin_visit_statuses.setdefault(visit_key, [])
+                while len(self._origin_visits[origin_url]) <= visit.visit:
                     self._origin_visits[origin_url].append(None)
 
-                self._origin_visits[origin_url][visit_id - 1] = visit
+                self._origin_visits[origin_url][visit.visit - 1] = visit
+                self._origin_visit_statuses[visit_key].append(visit_update)
 
-    def _convert_visit(self, visit):
-        if visit is None:
-            return
+                self._objects[visit_key].append(("origin_visit", None))
 
-        visit = visit.to_dict()
+    def _origin_visit_get_updated(self, origin: str, visit_id: int) -> OriginVisit:
+        """Merge origin visit and latest origin visit status
 
-        return visit
+        """
+        assert visit_id >= 1
+        visit = self._origin_visits[origin][visit_id - 1]
+        assert visit is not None
+        visit_key = (origin, visit_id)
+
+        visit_update = max(self._origin_visit_statuses[visit_key], key=lambda v: v.date)
+
+        return OriginVisit.from_dict(
+            {
+                # default to the values in visit
+                **visit.to_dict(),
+                # override with the last update
+                **visit_update.to_dict(),
+                # but keep the date of the creation of the origin visit
+                "date": visit.date,
+            }
+        )
 
     def origin_visit_get(
         self, origin: str, last_visit: Optional[int] = None, limit: Optional[int] = None
@@ -825,7 +949,9 @@ class InMemoryStorage:
                     continue
                 visit_id = visit.visit
 
-                yield self._convert_visit(self._origin_visits[origin_url][visit_id - 1])
+                visit_update = self._origin_visit_get_updated(origin_url, visit_id)
+                assert visit_update is not None
+                yield visit_update.to_dict()
 
     def origin_visit_find_by_date(
         self, origin: str, visit_date: datetime.datetime
@@ -834,7 +960,9 @@ class InMemoryStorage:
         if origin_url in self._origin_visits:
             visits = self._origin_visits[origin_url]
             visit = min(visits, key=lambda v: (abs(v.date - visit_date), -v.visit))
-            return self._convert_visit(visit)
+            visit_update = self._origin_visit_get_updated(origin, visit.visit)
+            assert visit_update is not None
+            return visit_update.to_dict()
         return None
 
     def origin_visit_get_by(self, origin: str, visit: int) -> Optional[Dict[str, Any]]:
@@ -842,7 +970,9 @@ class InMemoryStorage:
         if origin_url in self._origin_visits and visit <= len(
             self._origin_visits[origin_url]
         ):
-            return self._convert_visit(self._origin_visits[origin_url][visit - 1])
+            visit_update = self._origin_visit_get_updated(origin_url, visit)
+            assert visit_update is not None
+            return visit_update.to_dict()
         return None
 
     def origin_visit_get_latest(
@@ -855,13 +985,21 @@ class InMemoryStorage:
         if not ori:
             return None
         visits = self._origin_visits[ori.url]
+        visits = [
+            self._origin_visit_get_updated(visit.origin, visit.visit)
+            for visit in visits
+            if visit is not None
+        ]
+
         if allowed_statuses is not None:
             visits = [visit for visit in visits if visit.status in allowed_statuses]
         if require_snapshot:
             visits = [visit for visit in visits if visit.snapshot]
 
         visit = max(visits, key=lambda v: (v.date, v.visit), default=None)
-        return self._convert_visit(visit)
+        if visit is None:
+            return None
+        return visit.to_dict()
 
     def _select_random_origin_visit_by_type(self, type: str) -> str:
         while True:
@@ -877,8 +1015,10 @@ class InMemoryStorage:
         back_in_the_day = now() - timedelta(weeks=12)  # 3 months back
         # This should be enough for tests
         for visit in random_origin_visits:
-            if visit.date > back_in_the_day and visit.status == "full":
-                return visit.to_dict()
+            updated_visit = self._origin_visit_get_updated(url, visit.visit)
+            assert updated_visit is not None
+            if updated_visit.date > back_in_the_day and updated_visit.status == "full":
+                return updated_visit.to_dict()
         else:
             return None
 
@@ -906,71 +1046,108 @@ class InMemoryStorage:
     def refresh_stat_counters(self):
         pass
 
-    def origin_metadata_add(self, origin_url, ts, provider, tool, metadata):
+    def origin_metadata_add(
+        self,
+        origin_url: str,
+        discovery_date: datetime.datetime,
+        authority: Dict[str, Any],
+        fetcher: Dict[str, Any],
+        format: str,
+        metadata: bytes,
+    ) -> None:
         if not isinstance(origin_url, str):
-            raise TypeError("origin_id must be str, not %r" % (origin_url,))
-
-        if isinstance(ts, str):
-            ts = dateutil.parser.parse(ts)
+            raise StorageArgumentException(
+                "origin_id must be str, not %r" % (origin_url,)
+            )
+        if not isinstance(metadata, bytes):
+            raise StorageArgumentException(
+                "metadata must be bytes, not %r" % (metadata,)
+            )
+        authority_key = self._metadata_authority_key(authority)
+        if authority_key not in self._metadata_authorities:
+            raise StorageArgumentException(f"Unknown authority {authority}")
+        fetcher_key = self._metadata_fetcher_key(fetcher)
+        if fetcher_key not in self._metadata_fetchers:
+            raise StorageArgumentException(f"Unknown fetcher {fetcher}")
 
         origin_metadata = {
             "origin_url": origin_url,
-            "discovery_date": ts,
-            "tool_id": tool,
+            "discovery_date": discovery_date,
+            "authority": authority_key,
+            "fetcher": fetcher_key,
+            "format": format,
             "metadata": metadata,
-            "provider_id": provider,
         }
-        self._origin_metadata[origin_url].append(origin_metadata)
+        self._origin_metadata[origin_url][authority_key].add(origin_metadata)
         return None
 
-    def origin_metadata_get_by(self, origin_url, provider_type=None):
+    def origin_metadata_get(
+        self,
+        origin_url: str,
+        authority: Dict[str, str],
+        after: Optional[datetime.datetime] = None,
+        limit: Optional[int] = None,
+    ) -> List[Dict[str, Any]]:
         if not isinstance(origin_url, str):
             raise TypeError("origin_url must be str, not %r" % (origin_url,))
-        metadata = []
-        for item in self._origin_metadata[origin_url]:
-            item = copy.deepcopy(item)
-            provider = self.metadata_provider_get(item["provider_id"])
-            for attr_name in ("name", "type", "url"):
-                item["provider_" + attr_name] = provider["provider_" + attr_name]
-            metadata.append(item)
-        return metadata
 
-    def tool_add(self, tools):
-        inserted = []
-        for tool in tools:
-            key = self._tool_key(tool)
-            assert "id" not in tool
-            record = copy.deepcopy(tool)
-            record["id"] = key  # TODO: remove this
-            if key not in self._tools:
-                self._tools[key] = record
-            inserted.append(copy.deepcopy(self._tools[key]))
+        authority_key = self._metadata_authority_key(authority)
 
-        return inserted
+        if after is None:
+            entries = iter(self._origin_metadata[origin_url][authority_key])
+        else:
+            entries = self._origin_metadata[origin_url][authority_key].iter_from(after)
+        if limit:
+            entries = itertools.islice(entries, 0, limit)
 
-    def tool_get(self, tool):
-        return self._tools.get(self._tool_key(tool))
+        results = []
+        for entry in entries:
+            authority = self._metadata_authorities[entry["authority"]]
+            fetcher = self._metadata_fetchers[entry["fetcher"]]
+            results.append(
+                {
+                    **entry,
+                    "authority": {"type": authority["type"], "url": authority["url"],},
+                    "fetcher": {
+                        "name": fetcher["name"],
+                        "version": fetcher["version"],
+                    },
+                }
+            )
+        return results
 
-    def metadata_provider_add(
-        self, provider_name, provider_type, provider_url, metadata
-    ):
-        provider = {
-            "provider_name": provider_name,
-            "provider_type": provider_type,
-            "provider_url": provider_url,
+    def metadata_fetcher_add(
+        self, name: str, version: str, metadata: Dict[str, Any]
+    ) -> None:
+        fetcher = {
+            "name": name,
+            "version": version,
             "metadata": metadata,
         }
-        key = self._metadata_provider_key(provider)
-        provider["id"] = key
-        self._metadata_providers[key] = provider
-        return key
+        key = self._metadata_fetcher_key(fetcher)
+        if key not in self._metadata_fetchers:
+            self._metadata_fetchers[key] = fetcher
 
-    def metadata_provider_get(self, provider_id):
-        return self._metadata_providers.get(provider_id)
+    def metadata_fetcher_get(self, name: str, version: str) -> Optional[Dict[str, Any]]:
+        return self._metadata_fetchers.get(
+            self._metadata_fetcher_key({"name": name, "version": version})
+        )
 
-    def metadata_provider_get_by(self, provider):
-        key = self._metadata_provider_key(provider)
-        return self._metadata_providers.get(key)
+    def metadata_authority_add(
+        self, type: str, url: str, metadata: Dict[str, Any]
+    ) -> None:
+        authority = {
+            "type": type,
+            "url": url,
+            "metadata": metadata,
+        }
+        key = self._metadata_authority_key(authority)
+        self._metadata_authorities[key] = authority
+
+    def metadata_authority_get(self, type: str, url: str) -> Optional[Dict[str, Any]]:
+        return self._metadata_authorities.get(
+            self._metadata_authority_key({"type": type, "url": url})
+        )
 
     def _get_origin_url(self, origin):
         if isinstance(origin, str):
@@ -997,16 +1174,12 @@ class InMemoryStorage:
         return tuple((key, content.get(key)) for key in sorted(DEFAULT_ALGORITHMS))
 
     @staticmethod
-    def _tool_key(tool):
-        return "%r %r %r" % (
-            tool["name"],
-            tool["version"],
-            tuple(sorted(tool["configuration"].items())),
-        )
+    def _metadata_fetcher_key(fetcher: Dict) -> Hashable:
+        return (fetcher["name"], fetcher["version"])
 
     @staticmethod
-    def _metadata_provider_key(provider):
-        return "%r %r" % (provider["provider_name"], provider["provider_url"])
+    def _metadata_authority_key(authority: Dict) -> Hashable:
+        return (authority["type"], authority["url"])
 
     def diff_directories(self, from_dir, to_dir, track_renaming=False):
         raise NotImplementedError("InMemoryStorage.diff_directories")
