@@ -17,7 +17,7 @@ comment on column dbversion.description is 'Release description';
 
 -- latest schema version
 insert into dbversion(version, release, description)
-      values(146, now(), 'Work In Progress');
+      values(149, now(), 'Work In Progress');
 
 -- a SHA1 checksum
 create domain sha1 as bytea check (length(value) = 20);
@@ -239,23 +239,22 @@ create table revision
   committer_date_neg_utc_offset boolean
 );
 
-comment on table revision is 'Revision represents the state of a source code tree at a
- specific point in time';
-comment on column revision.id is 'Git id of sha1 checksum';
-comment on column revision.date is 'Timestamp when revision was authored';
-comment on column revision.date_offset is 'Authored timestamp offset from UTC';
-comment on column revision.committer_date is 'Timestamp when revision was committed';
-comment on column revision.committer_date_offset is 'Committed timestamp offset from UTC';
-comment on column revision.type is 'Possible revision types (''git'', ''tar'', ''dsc'', ''svn'', ''hg'')';
+comment on table revision is 'A revision represents the state of a source code tree at a specific point in time';
+comment on column revision.id is 'Git-style SHA1 commit identifier';
+comment on column revision.date is 'Author timestamp as UNIX epoch';
+comment on column revision.date_offset is 'Author timestamp timezone, as minute offsets from UTC';
+comment on column revision.date_neg_utc_offset is 'True indicates a -0 UTC offset on author timestamp';
+comment on column revision.committer_date is 'Committer timestamp as UNIX epoch';
+comment on column revision.committer_date_offset is 'Committer timestamp timezone, as minute offsets from UTC';
+comment on column revision.committer_date_neg_utc_offset is 'True indicates a -0 UTC offset on committer timestamp';
+comment on column revision.type is 'Type of revision';
 comment on column revision.directory is 'Directory identifier';
-comment on column revision.message is 'Revision message';
-comment on column revision.author is 'Author identifier';
-comment on column revision.committer is 'Committer identifier';
-comment on column revision.synthetic is 'true iff revision has been created by Software Heritage';
-comment on column revision.metadata is 'extra metadata (tarball checksums, extra commit information, etc...)';
-comment on column revision.object_id is 'Object identifier';
-comment on column revision.date_neg_utc_offset is 'True indicates -0 UTC offset for author timestamp';
-comment on column revision.committer_date_neg_utc_offset is 'True indicates -0 UTC offset for committer timestamp';
+comment on column revision.message is 'Commit message';
+comment on column revision.author is 'Author identity';
+comment on column revision.committer is 'Committer identity';
+comment on column revision.synthetic is 'True iff revision has been synthesized by Software Heritage';
+comment on column revision.metadata is 'Extra revision metadata';
+comment on column revision.object_id is 'Non-intrinsic, sequential object identifier';
 
 
 -- either this table or the sha1_git[] column on the revision table
@@ -282,7 +281,8 @@ create table origin_visit
   visit        bigint not null,
   date         timestamptz not null,
   type         text not null,
-  status       origin_visit_status not null,
+  -- remove those when done migrating the schema
+  status       origin_visit_state not null,
   metadata     jsonb,
   snapshot     sha1_git
 );
@@ -291,9 +291,29 @@ comment on column origin_visit.origin is 'Visited origin';
 comment on column origin_visit.visit is 'Sequential visit number for the origin';
 comment on column origin_visit.date is 'Visit timestamp';
 comment on column origin_visit.type is 'Type of loader that did the visit (hg, git, ...)';
-comment on column origin_visit.status is 'Visit result';
-comment on column origin_visit.metadata is 'Origin metadata at visit time';
-comment on column origin_visit.snapshot is 'Origin snapshot at visit time';
+comment on column origin_visit.status is '(Deprecated) Visit status';
+comment on column origin_visit.metadata is '(Deprecated) Optional origin visit metadata';
+comment on column origin_visit.snapshot is '(Deprecated) Optional snapshot of the origin visit. It can be partial.';
+
+
+-- Crawling history of software origin visits by Software Heritage. Each
+-- visit see its history change through new origin visit status updates
+create table origin_visit_status
+(
+  origin   bigint not null,
+  visit    bigint not null,
+  date     timestamptz not null,
+  status   origin_visit_state not null,
+  metadata jsonb,
+  snapshot sha1_git
+);
+
+comment on column origin_visit_status.origin is 'Origin concerned by the visit update';
+comment on column origin_visit_status.visit is 'Visit concerned by the visit update';
+comment on column origin_visit_status.date is 'Visit update timestamp';
+comment on column origin_visit_status.status is 'Visit status (ongoing, failed, full)';
+comment on column origin_visit_status.metadata is 'Optional origin visit metadata';
+comment on column origin_visit_status.snapshot is 'Optional, possibly partial, snapshot of the origin visit. It can be partial.';
 
 
 -- A snapshot represents the entire state of a software origin as crawled by
@@ -377,35 +397,34 @@ comment on column release.target_type is 'Object type (''content'', ''directory'
 comment on column release.date_neg_utc_offset is 'True indicates -0 UTC offset for release timestamp';
 
 -- Tools
-create table tool
+create table metadata_fetcher
 (
-  id serial not null,
-  name text not null,
-  version text not null,
-  configuration jsonb
+  id            serial  not null,
+  name          text    not null,
+  version       text    not null,
+  metadata      jsonb   not null
 );
 
-comment on table tool is 'Tool information';
-comment on column tool.id is 'Tool identifier';
-comment on column tool.version is 'Tool name';
-comment on column tool.version is 'Tool version';
-comment on column tool.configuration is 'Tool configuration: command line, flags, etc...';
+comment on table metadata_fetcher is 'Tools used to retrieve metadata';
+comment on column metadata_fetcher.id is 'Internal identifier of the fetcher';
+comment on column metadata_fetcher.name is 'Fetcher name';
+comment on column metadata_fetcher.version is 'Fetcher version';
+comment on column metadata_fetcher.metadata is 'Extra information about the fetcher';
 
 
-create table metadata_provider
+create table metadata_authority
 (
-  id            serial not null,
-  provider_name text   not null,
-  provider_type text   not null,
-  provider_url  text,
-  metadata      jsonb
+  id            serial  not null,
+  type          text    not null,
+  url           text    not null,
+  metadata      jsonb   not null
 );
 
-comment on table metadata_provider is 'Metadata provider information';
-comment on column metadata_provider.id is 'Provider''s identifier';
-comment on column metadata_provider.provider_name is 'Provider''s name';
-comment on column metadata_provider.provider_url is 'Provider''s url';
-comment on column metadata_provider.metadata is 'Other metadata about provider';
+comment on table metadata_authority is 'Metadata authority information';
+comment on column metadata_authority.id is 'Internal identifier of the authority';
+comment on column metadata_authority.type is 'Type of authority (deposit/forge/registry)';
+comment on column metadata_authority.url is 'Authority''s uri';
+comment on column metadata_authority.metadata is 'Other metadata about authority';
 
 
 -- Discovery of metadata during a listing, loading, deposit or external_catalog of an origin
@@ -415,18 +434,20 @@ create table origin_metadata
   id             bigserial     not null,  -- PK internal object identifier
   origin_id      bigint        not null,  -- references origin(id)
   discovery_date timestamptz   not null,  -- when it was extracted
-  provider_id    bigint        not null,  -- ex: 'hal', 'lister-github', 'loader-github'
-  tool_id        bigint        not null,
-  metadata       jsonb         not null
+  authority_id   bigint        not null,
+  fetcher_id     bigint        not null,
+  format         text          not null,
+  metadata       bytea         not null
 );
 
 comment on table origin_metadata is 'keeps all metadata found concerning an origin';
 comment on column origin_metadata.id is 'the origin_metadata object''s id';
 comment on column origin_metadata.origin_id is 'the origin id for which the metadata was found';
 comment on column origin_metadata.discovery_date is 'the date of retrieval';
-comment on column origin_metadata.provider_id is 'the metadata provider: github, openhub, deposit, etc.';
-comment on column origin_metadata.tool_id is 'the tool used for extracting metadata: lister-github, etc.';
-comment on column origin_metadata.metadata is 'metadata in json format but with original terms';
+comment on column origin_metadata.authority_id is 'the metadata provider: github, openhub, deposit, etc.';
+comment on column origin_metadata.fetcher_id is 'the tool used for extracting metadata: loaders, crawlers, etc.';
+comment on column origin_metadata.format is 'name of the format of metadata, used by readers to interpret it.';
+comment on column origin_metadata.metadata is 'original metadata in opaque format';
 
 
 -- Keep a cache of object counts
