@@ -6,7 +6,6 @@
 import contextlib
 import datetime
 import itertools
-import json
 
 from collections import defaultdict
 from contextlib import contextmanager
@@ -23,6 +22,7 @@ from swh.model.model import (
     Directory,
     Origin,
     OriginVisit,
+    OriginVisitStatus,
     Revision,
     Release,
     SkippedContent,
@@ -706,7 +706,7 @@ class Storage:
                         cur,
                     )
 
-                self.journal_writer.snapshot_add(snapshot)
+                self.journal_writer.snapshot_add([snapshot])
 
                 db.snapshot_add(snapshot.id, cur)
                 count += 1
@@ -843,6 +843,7 @@ class Storage:
         with convert_validation_exceptions():
             visit_id = db.origin_visit_add(origin_url, date, type, cur=cur)
 
+        status = "ongoing"
         # We can write to the journal only after inserting to the
         # DB, because we want the id of the visit
         visit = OriginVisit.from_dict(
@@ -851,16 +852,39 @@ class Storage:
                 "date": date,
                 "type": type,
                 "visit": visit_id,
-                "status": "ongoing",
+                # TODO: Remove when we remove those fields from the model
+                "status": status,
                 "metadata": None,
                 "snapshot": None,
             }
         )
-        self.journal_writer.origin_visit_add(visit)
+
+        with convert_validation_exceptions():
+            visit_status = OriginVisitStatus(
+                origin=origin_url,
+                visit=visit_id,
+                date=date,
+                status=status,
+                snapshot=None,
+                metadata=None,
+            )
+        self._origin_visit_status_add(visit_status, db=db, cur=cur)
+
+        self.journal_writer.origin_visit_add([visit])
 
         send_metric("origin_visit:add", count=1, method_name="origin_visit")
         return visit
 
+    def _origin_visit_status_add(
+        self, origin_visit_status: OriginVisitStatus, db, cur
+    ) -> None:
+        """Add an origin visit status"""
+        db.origin_visit_status_add(origin_visit_status, cur=cur)
+        # TODO: write to the journal the origin visit status
+        send_metric(
+            "origin_visit_status:add", count=1, method_name="origin_visit_status"
+        )
+
     @timed
     @db_transaction()
     def origin_visit_update(
@@ -897,10 +921,72 @@ class Storage:
         if updates:
             with convert_validation_exceptions():
                 updated_visit = OriginVisit.from_dict({**visit, **updates})
-            self.journal_writer.origin_visit_update(updated_visit)
+            self.journal_writer.origin_visit_update([updated_visit])
+
+            # Write updates to origin visit (backward compatibility)
+            db.origin_visit_update(origin, visit_id, updates)
+
+            # Add new origin visit status
+            last_visit_status = self._origin_visit_get_updated(
+                origin, visit_id, db=db, cur=cur
+            )
+            assert last_visit_status is not None
 
             with convert_validation_exceptions():
-                db.origin_visit_update(origin_url, visit_id, updates, cur)
+                visit_status = OriginVisitStatus(
+                    origin=origin_url,
+                    visit=visit_id,
+                    date=date or now(),
+                    status=status,
+                    snapshot=snapshot or last_visit_status["snapshot"],
+                    metadata=metadata or last_visit_status["metadata"],
+                )
+            self._origin_visit_status_add(visit_status, db=db, cur=cur)
+
+    def _origin_visit_get_updated(
+        self, origin: str, visit_id: int, db, cur
+    ) -> Optional[Dict[str, Any]]:
+        """Retrieve origin visit and latest origin visit status and merge them
+        into an origin visit.
+
+        """
+        row_visit = db.origin_visit_get(origin, visit_id)
+        if row_visit is None:
+            return None
+        visit = dict(zip(db.origin_visit_get_cols, row_visit))
+        return self._origin_visit_apply_update(visit, db=db, cur=cur)
+
+    def _origin_visit_apply_update(
+        self, visit: Dict[str, Any], db, cur=None
+    ) -> Dict[str, Any]:
+        """Retrieve the latest visit status information for the origin visit.
+        Then merge it with the visit and return it.
+
+        """
+        visit_status = db.origin_visit_status_get_latest(
+            visit["origin"], visit["visit"]
+        )
+        return self._origin_visit_merge(visit, visit_status)
+
+    def _origin_visit_merge(
+        self, visit: Dict[str, Any], visit_status: Dict[str, Any]
+    ) -> Dict[str, Any]:
+        """Merge origin_visit and origin_visit_status together.
+
+        """
+        return OriginVisit.from_dict(
+            {
+                # default to the values in visit
+                **visit,
+                # override with the last update
+                **visit_status,
+                # visit['origin'] is the URL (via a join), while
+                # visit_status['origin'] is only an id.
+                "origin": visit["origin"],
+                # but keep the date of the creation of the origin visit
+                "date": visit["date"],
+            }
+        ).to_dict()
 
     @timed
     @db_transaction()
@@ -915,7 +1001,18 @@ class Storage:
 
         for visit in visits:
             # TODO: upsert them all in a single query
+            assert visit.visit is not None
             db.origin_visit_upsert(visit, cur=cur)
+            with convert_validation_exceptions():
+                visit_status = OriginVisitStatus(
+                    origin=visit.origin,
+                    visit=visit.visit,
+                    date=now(),
+                    status=visit.status,
+                    snapshot=visit.snapshot,
+                    metadata=visit.metadata,
+                )
+            db.origin_visit_status_add(visit_status, cur=cur)
 
     @timed
     @db_transaction_generator(statement_timeout=500)
@@ -938,9 +1035,9 @@ class Storage:
     def origin_visit_find_by_date(
         self, origin: str, visit_date: datetime.datetime, db=None, cur=None
     ) -> Optional[Dict[str, Any]]:
-        line = db.origin_visit_find_by_date(origin, visit_date, cur=cur)
-        if line:
-            return dict(zip(db.origin_visit_get_cols, line))
+        visit = db.origin_visit_find_by_date(origin, visit_date, cur=cur)
+        if visit:
+            return visit
         return None
 
     @timed
@@ -1104,7 +1201,7 @@ class Storage:
         if origin_url:
             return origin_url
 
-        self.journal_writer.origin_add_one(origin)
+        self.journal_writer.origin_add([origin])
 
         url = db.origin_add(origin.url, cur)
         send_metric("origin:add", count=1, method_name="origin_add_one")
@@ -1138,72 +1235,110 @@ class Storage:
     @timed
     @db_transaction()
     def origin_metadata_add(
-        self, origin_url, ts, provider, tool, metadata, db=None, cur=None
-    ):
-        if isinstance(ts, str):
-            ts = dateutil.parser.parse(ts)
-
-        db.origin_metadata_add(origin_url, ts, provider, tool, metadata, cur)
+        self,
+        origin_url: str,
+        discovery_date: datetime.datetime,
+        authority: Dict[str, Any],
+        fetcher: Dict[str, Any],
+        format: str,
+        metadata: bytes,
+        db=None,
+        cur=None,
+    ) -> None:
+        authority_id = db.metadata_authority_get_id(
+            authority["type"], authority["url"], cur
+        )
+        if not authority_id:
+            raise StorageArgumentException(f"Unknown authority {authority}")
+        fetcher_id = db.metadata_fetcher_get_id(
+            fetcher["name"], fetcher["version"], cur
+        )
+        if not fetcher_id:
+            raise StorageArgumentException(f"Unknown fetcher {fetcher}")
+        try:
+            db.origin_metadata_add(
+                origin_url,
+                discovery_date,
+                authority_id,
+                fetcher_id,
+                format,
+                metadata,
+                cur,
+            )
+        except psycopg2.ProgrammingError as e:
+            raise StorageArgumentException(*e.args)
         send_metric("origin_metadata:add", count=1, method_name="origin_metadata_add")
 
     @timed
-    @db_transaction_generator(statement_timeout=500)
-    def origin_metadata_get_by(self, origin_url, provider_type=None, db=None, cur=None):
-        for line in db.origin_metadata_get_by(origin_url, provider_type, cur):
-            yield dict(zip(db.origin_metadata_get_cols, line))
-
-    @timed
-    @db_transaction()
-    def tool_add(self, tools, db=None, cur=None):
-        db.mktemp_tool(cur)
-        with convert_validation_exceptions():
-            db.copy_to(tools, "tmp_tool", ["name", "version", "configuration"], cur)
-            tools = db.tool_add_from_temp(cur)
-
-        results = [dict(zip(db.tool_cols, line)) for line in tools]
-        send_metric("tool:add", count=len(results), method_name="tool_add")
+    @db_transaction(statement_timeout=500)
+    def origin_metadata_get(
+        self,
+        origin_url: str,
+        authority: Dict[str, str],
+        after: Optional[datetime.datetime] = None,
+        limit: Optional[int] = None,
+        db=None,
+        cur=None,
+    ) -> List[Dict[str, Any]]:
+        authority_id = db.metadata_authority_get_id(
+            authority["type"], authority["url"], cur
+        )
+        if not authority_id:
+            return []
+        results = []
+        for line in db.origin_metadata_get(origin_url, authority_id, after, limit, cur):
+            row = dict(zip(db.origin_metadata_get_cols, line))
+            results.append(
+                {
+                    "origin_url": row.pop("origin.url"),
+                    "authority": {
+                        "type": row.pop("metadata_authority.type"),
+                        "url": row.pop("metadata_authority.url"),
+                    },
+                    "fetcher": {
+                        "name": row.pop("metadata_fetcher.name"),
+                        "version": row.pop("metadata_fetcher.version"),
+                    },
+                    **row,
+                }
+            )
         return results
 
+    @timed
+    @db_transaction()
+    def metadata_fetcher_add(
+        self, name: str, version: str, metadata: Dict[str, Any], db=None, cur=None
+    ) -> None:
+        db.metadata_fetcher_add(name, version, metadata)
+        send_metric("metadata_fetcher:add", count=1, method_name="metadata_fetcher")
+
     @timed
     @db_transaction(statement_timeout=500)
-    def tool_get(self, tool, db=None, cur=None):
-        tool_conf = tool["configuration"]
-        if isinstance(tool_conf, dict):
-            tool_conf = json.dumps(tool_conf)
-
-        idx = db.tool_get(tool["name"], tool["version"], tool_conf)
-        if not idx:
+    def metadata_fetcher_get(
+        self, name: str, version: str, db=None, cur=None
+    ) -> Optional[Dict[str, Any]]:
+        row = db.metadata_fetcher_get(name, version, cur=cur)
+        if not row:
             return None
-        return dict(zip(db.tool_cols, idx))
+        return dict(zip(db.metadata_fetcher_cols, row))
 
     @timed
     @db_transaction()
-    def metadata_provider_add(
-        self, provider_name, provider_type, provider_url, metadata, db=None, cur=None
-    ):
-        result = db.metadata_provider_add(
-            provider_name, provider_type, provider_url, metadata, cur
-        )
-        send_metric("metadata_provider:add", count=1, method_name="metadata_provider")
-        return result
+    def metadata_authority_add(
+        self, type: str, url: str, metadata: Dict[str, Any], db=None, cur=None
+    ) -> None:
+        db.metadata_authority_add(type, url, metadata, cur)
+        send_metric("metadata_authority:add", count=1, method_name="metadata_authority")
 
     @timed
     @db_transaction()
-    def metadata_provider_get(self, provider_id, db=None, cur=None):
-        result = db.metadata_provider_get(provider_id)
-        if not result:
+    def metadata_authority_get(
+        self, type: str, url: str, db=None, cur=None
+    ) -> Optional[Dict[str, Any]]:
+        row = db.metadata_authority_get(type, url, cur=cur)
+        if not row:
             return None
-        return dict(zip(db.metadata_provider_cols, result))
-
-    @timed
-    @db_transaction()
-    def metadata_provider_get_by(self, provider, db=None, cur=None):
-        result = db.metadata_provider_get_by(
-            provider["provider_name"], provider["provider_url"]
-        )
-        if not result:
-            return None
-        return dict(zip(db.metadata_provider_cols, result))
+        return dict(zip(db.metadata_authority_cols, row))
 
     @timed
     def diff_directories(self, from_dir, to_dir, track_renaming=False):
