@@ -3,7 +3,6 @@
 # License: GNU General Public License version 3, or any later version
 # See top-level LICENSE file for more information
 
-from typing import Dict
 from unittest.mock import call
 
 import psycopg2
@@ -25,6 +24,20 @@ from swh.storage.exc import HashCollision, StorageArgumentException
 from .storage_data import date_visit1
 
 
+@pytest.fixture
+def monkeypatch_sleep(monkeypatch, swh_storage):
+    """In test context, we don't want to wait, make test faster
+
+    """
+    from swh.storage.retry import RetryingProxyStorage
+
+    for method_name, method in RetryingProxyStorage.__dict__.items():
+        if "_add" in method_name or "_update" in method_name:
+            monkeypatch.setattr(method.retry, "sleep", lambda x: None)
+
+    return monkeypatch
+
+
 @pytest.fixture
 def fake_hash_collision(sample_data):
     return HashCollision("sha1", "38762cf7f55934b34d179ae6a4c80cadccbb7f0a", [])
@@ -59,7 +72,7 @@ def test_retrying_proxy_storage_content_add(swh_storage, sample_data):
 
 
 def test_retrying_proxy_storage_content_add_with_retry(
-    swh_storage, sample_data, mocker, fake_hash_collision
+    monkeypatch_sleep, swh_storage, sample_data, mocker, fake_hash_collision,
 ):
     """Multiple retries for hash collision and psycopg2 error but finally ok
 
@@ -73,9 +86,6 @@ def test_retrying_proxy_storage_content_add_with_retry(
         # ok then!
         {"content:add": 1},
     ]
-    mock_sleep = mocker.patch(
-        "swh.storage.retry.RetryingProxyStorage" ".content_add.retry.sleep"
-    )
 
     sample_content = sample_data["content"][0]
 
@@ -92,7 +102,6 @@ def test_retrying_proxy_storage_content_add_with_retry(
             call([Content.from_dict(sample_content)]),
         ]
     )
-    assert mock_sleep.call_count == 2
 
 
 def test_retrying_proxy_swh_storage_content_add_failure(
@@ -136,7 +145,7 @@ def test_retrying_proxy_storage_content_add_metadata(swh_storage, sample_data):
 
 
 def test_retrying_proxy_storage_content_add_metadata_with_retry(
-    swh_storage, sample_data, mocker, fake_hash_collision
+    monkeypatch_sleep, swh_storage, sample_data, mocker, fake_hash_collision
 ):
     """Multiple retries for hash collision and psycopg2 error but finally ok
 
@@ -152,9 +161,6 @@ def test_retrying_proxy_storage_content_add_metadata_with_retry(
         # ok then!
         {"content:add": 1},
     ]
-    mock_sleep = mocker.patch(
-        "swh.storage.retry.RetryingProxyStorage" ".content_add_metadata.retry.sleep"
-    )
 
     sample_content = sample_data["content_metadata"][0]
 
@@ -168,7 +174,6 @@ def test_retrying_proxy_storage_content_add_metadata_with_retry(
             call([Content.from_dict(sample_content)]),
         ]
     )
-    assert mock_sleep.call_count == 2
 
 
 def test_retrying_proxy_swh_storage_content_add_metadata_failure(
@@ -215,7 +220,7 @@ def test_retrying_proxy_storage_skipped_content_add(swh_storage, sample_data):
 
 
 def test_retrying_proxy_storage_skipped_content_add_with_retry(
-    swh_storage, sample_data, mocker, fake_hash_collision
+    monkeypatch_sleep, swh_storage, sample_data, mocker, fake_hash_collision
 ):
     """Multiple retries for hash collision and psycopg2 error but finally ok
 
@@ -230,9 +235,6 @@ def test_retrying_proxy_storage_skipped_content_add_with_retry(
         # ok then!
         {"skipped_content:add": 1},
     ]
-    mock_sleep = mocker.patch(
-        "swh.storage.retry.RetryingProxyStorage" ".skipped_content_add.retry.sleep"
-    )
 
     sample_content = sample_data["skipped_content"][0]
 
@@ -246,7 +248,6 @@ def test_retrying_proxy_storage_skipped_content_add_with_retry(
             call([SkippedContent.from_dict(sample_content)]),
         ]
     )
-    assert mock_sleep.call_count == 2
 
 
 def test_retrying_proxy_swh_storage_skipped_content_add_failure(
@@ -293,7 +294,7 @@ def test_retrying_proxy_swh_storage_origin_add_one(swh_storage, sample_data):
 
 
 def test_retrying_proxy_swh_storage_origin_add_one_retry(
-    swh_storage, sample_data, mocker, fake_hash_collision
+    monkeypatch_sleep, swh_storage, sample_data, mocker, fake_hash_collision
 ):
     """Multiple retries for hash collision and psycopg2 error but finally ok
 
@@ -308,9 +309,6 @@ def test_retrying_proxy_swh_storage_origin_add_one_retry(
         # ok then!
         sample_origin["url"],
     ]
-    mock_sleep = mocker.patch(
-        "swh.storage.retry.RetryingProxyStorage" ".origin_add_one.retry.sleep"
-    )
 
     origin = swh_storage.origin_get(sample_origin)
     assert not origin
@@ -325,7 +323,6 @@ def test_retrying_proxy_swh_storage_origin_add_one_retry(
             call(Origin.from_dict(sample_origin)),
         ]
     )
-    assert mock_sleep.call_count == 2
 
 
 def test_retrying_proxy_swh_storage_origin_add_one_failure(
@@ -368,7 +365,7 @@ def test_retrying_proxy_swh_storage_origin_visit_add(swh_storage, sample_data):
 
 
 def test_retrying_proxy_swh_storage_origin_visit_add_retry(
-    swh_storage, sample_data, mocker, fake_hash_collision
+    monkeypatch_sleep, swh_storage, sample_data, mocker, fake_hash_collision
 ):
     """Multiple retries for hash collision and psycopg2 error but finally ok
 
@@ -385,9 +382,6 @@ def test_retrying_proxy_swh_storage_origin_visit_add_retry(
         # ok then!
         {"origin": origin_url, "visit": 1},
     ]
-    mock_sleep = mocker.patch(
-        "swh.storage.retry.RetryingProxyStorage" ".origin_visit_add.retry.sleep"
-    )
 
     origin = list(swh_storage.origin_visit_get(origin_url))
     assert not origin
@@ -402,7 +396,6 @@ def test_retrying_proxy_swh_storage_origin_visit_add_retry(
             call(origin_url, date_visit1, "git"),
         ]
     )
-    assert mock_sleep.call_count == 2
 
 
 def test_retrying_proxy_swh_storage_origin_visit_add_failure(
@@ -427,164 +420,152 @@ def test_retrying_proxy_swh_storage_origin_visit_add_failure(
     )
 
 
-def test_retrying_proxy_storage_tool_add(swh_storage, sample_data):
-    """Standard tool_add works as before
+def test_retrying_proxy_storage_metadata_fetcher_add(swh_storage, sample_data):
+    """Standard metadata_fetcher_add works as before
 
     """
-    sample_tool = sample_data["tool"][0]
+    fetcher = sample_data["fetcher"][0]
 
-    tool = swh_storage.tool_get(sample_tool)
-    assert not tool
+    metadata_fetcher = swh_storage.metadata_fetcher_get(
+        fetcher["name"], fetcher["version"]
+    )
+    assert not metadata_fetcher
 
-    tools = swh_storage.tool_add([sample_tool])
-    assert tools
-    tool = tools[0]
-    tool.pop("id")
-    assert tool == sample_tool
+    swh_storage.metadata_fetcher_add(**fetcher)
 
-    tool = swh_storage.tool_get(sample_tool)
-    tool.pop("id")
-    assert tool == sample_tool
+    actual_fetcher = swh_storage.metadata_fetcher_get(
+        fetcher["name"], fetcher["version"]
+    )
+    assert actual_fetcher == fetcher
 
 
-def test_retrying_proxy_storage_tool_add_with_retry(
-    swh_storage, sample_data, mocker, fake_hash_collision
+def test_retrying_proxy_storage_metadata_fetcher_add_with_retry(
+    monkeypatch_sleep, swh_storage, sample_data, mocker, fake_hash_collision
 ):
     """Multiple retries for hash collision and psycopg2 error but finally ok
 
     """
-    sample_tool = sample_data["tool"][0]
-    mock_memory = mocker.patch("swh.storage.in_memory.InMemoryStorage.tool_add")
-    mock_memory.side_effect = [
-        # first try goes ko
-        fake_hash_collision,
-        # second try goes ko
-        psycopg2.IntegrityError("tool already inserted"),
-        # ok then!
-        [sample_tool],
-    ]
-    mock_sleep = mocker.patch(
-        "swh.storage.retry.RetryingProxyStorage" ".tool_add.retry.sleep"
-    )
-
-    tool = swh_storage.tool_get(sample_tool)
-    assert not tool
-
-    tools = swh_storage.tool_add([sample_tool])
-    assert tools == [sample_tool]
-
-    mock_memory.assert_has_calls(
-        [call([sample_tool]), call([sample_tool]), call([sample_tool]),]
-    )
-    assert mock_sleep.call_count == 2
-
-
-def test_retrying_proxy_swh_storage_tool_add_failure(swh_storage, sample_data, mocker):
-    """Unfiltered errors are raising without retry
-
-    """
-    mock_memory = mocker.patch("swh.storage.in_memory.InMemoryStorage.tool_add")
-    mock_memory.side_effect = StorageArgumentException("Refuse to add tool always!")
-
-    sample_tool = sample_data["tool"][0]
-
-    tool = swh_storage.tool_get(sample_tool)
-    assert not tool
-
-    with pytest.raises(StorageArgumentException, match="Refuse to add"):
-        swh_storage.tool_add([sample_tool])
-
-    assert mock_memory.call_count == 1
-
-
-def to_provider(provider: Dict) -> Dict:
-    return {
-        "provider_name": provider["name"],
-        "provider_url": provider["url"],
-        "provider_type": provider["type"],
-        "metadata": provider["metadata"],
-    }
-
-
-def test_retrying_proxy_storage_metadata_provider_add(swh_storage, sample_data):
-    """Standard metadata_provider_add works as before
-
-    """
-    provider = sample_data["provider"][0]
-    provider_get = to_provider(provider)
-
-    provider = swh_storage.metadata_provider_get_by(provider_get)
-    assert not provider
-
-    provider_id = swh_storage.metadata_provider_add(**provider_get)
-    assert provider_id
-
-    actual_provider = swh_storage.metadata_provider_get(provider_id)
-    assert actual_provider
-    actual_provider_id = actual_provider.pop("id")
-    assert actual_provider_id == provider_id
-    assert actual_provider == provider_get
-
-
-def test_retrying_proxy_storage_metadata_provider_add_with_retry(
-    swh_storage, sample_data, mocker, fake_hash_collision
-):
-    """Multiple retries for hash collision and psycopg2 error but finally ok
-
-    """
-    provider = sample_data["provider"][0]
-    provider_get = to_provider(provider)
-
+    fetcher = sample_data["fetcher"][0]
     mock_memory = mocker.patch(
-        "swh.storage.in_memory.InMemoryStorage.metadata_provider_add"
+        "swh.storage.in_memory.InMemoryStorage.metadata_fetcher_add"
     )
     mock_memory.side_effect = [
         # first try goes ko
         fake_hash_collision,
         # second try goes ko
-        psycopg2.IntegrityError("provider_id already inserted"),
+        psycopg2.IntegrityError("metadata_fetcher already inserted"),
         # ok then!
-        "provider_id",
+        [fetcher],
     ]
-    mock_sleep = mocker.patch(
-        "swh.storage.retry.RetryingProxyStorage" ".metadata_provider_add.retry.sleep"
+
+    actual_fetcher = swh_storage.metadata_fetcher_get(
+        fetcher["name"], fetcher["version"]
     )
+    assert not actual_fetcher
 
-    provider = swh_storage.metadata_provider_get_by(provider_get)
-    assert not provider
+    swh_storage.metadata_fetcher_add(**fetcher)
 
-    provider_id = swh_storage.metadata_provider_add(**provider_get)
-    assert provider_id == "provider_id"
-
-    provider_arg_names = ("provider_name", "provider_type", "provider_url", "metadata")
-    provider_args = [provider_get[key] for key in provider_arg_names]
     mock_memory.assert_has_calls(
-        [call(*provider_args), call(*provider_args), call(*provider_args),]
+        [
+            call(fetcher["name"], fetcher["version"], fetcher["metadata"]),
+            call(fetcher["name"], fetcher["version"], fetcher["metadata"]),
+            call(fetcher["name"], fetcher["version"], fetcher["metadata"]),
+        ]
     )
-    assert mock_sleep.call_count == 2
 
 
-def test_retrying_proxy_swh_storage_metadata_provider_add_failure(
+def test_retrying_proxy_swh_storage_metadata_fetcher_add_failure(
     swh_storage, sample_data, mocker
 ):
     """Unfiltered errors are raising without retry
 
     """
     mock_memory = mocker.patch(
-        "swh.storage.in_memory.InMemoryStorage.metadata_provider_add"
+        "swh.storage.in_memory.InMemoryStorage.metadata_fetcher_add"
     )
     mock_memory.side_effect = StorageArgumentException(
-        "Refuse to add provider_id always!"
+        "Refuse to add metadata_fetcher always!"
     )
 
-    provider = sample_data["provider"][0]
-    provider_get = to_provider(provider)
+    fetcher = sample_data["fetcher"][0]
 
-    provider_id = swh_storage.metadata_provider_get_by(provider_get)
-    assert not provider_id
+    actual_fetcher = swh_storage.metadata_fetcher_get(
+        fetcher["name"], fetcher["version"]
+    )
+    assert not actual_fetcher
 
     with pytest.raises(StorageArgumentException, match="Refuse to add"):
-        swh_storage.metadata_provider_add(**provider_get)
+        swh_storage.metadata_fetcher_add(**fetcher)
+
+    assert mock_memory.call_count == 1
+
+
+def test_retrying_proxy_storage_metadata_authority_add(swh_storage, sample_data):
+    """Standard metadata_authority_add works as before
+
+    """
+    authority = sample_data["authority"][0]
+
+    assert not swh_storage.metadata_authority_get(authority["type"], authority["url"])
+
+    swh_storage.metadata_authority_add(**authority)
+
+    actual_authority = swh_storage.metadata_authority_get(
+        authority["type"], authority["url"]
+    )
+    assert actual_authority == authority
+
+
+def test_retrying_proxy_storage_metadata_authority_add_with_retry(
+    monkeypatch_sleep, swh_storage, sample_data, mocker, fake_hash_collision
+):
+    """Multiple retries for hash collision and psycopg2 error but finally ok
+
+    """
+    authority = sample_data["authority"][0]
+
+    mock_memory = mocker.patch(
+        "swh.storage.in_memory.InMemoryStorage.metadata_authority_add"
+    )
+    mock_memory.side_effect = [
+        # first try goes ko
+        fake_hash_collision,
+        # second try goes ko
+        psycopg2.IntegrityError("foo bar"),
+        # ok then!
+        None,
+    ]
+
+    assert not swh_storage.metadata_authority_get(authority["type"], authority["url"])
+
+    swh_storage.metadata_authority_add(**authority)
+
+    authority_arg_names = ("type", "url", "metadata")
+    authority_args = [authority[key] for key in authority_arg_names]
+    mock_memory.assert_has_calls(
+        [call(*authority_args), call(*authority_args), call(*authority_args),]
+    )
+
+
+def test_retrying_proxy_swh_storage_metadata_authority_add_failure(
+    swh_storage, sample_data, mocker
+):
+    """Unfiltered errors are raising without retry
+
+    """
+    mock_memory = mocker.patch(
+        "swh.storage.in_memory.InMemoryStorage.metadata_authority_add"
+    )
+    mock_memory.side_effect = StorageArgumentException(
+        "Refuse to add authority_id always!"
+    )
+
+    authority = sample_data["authority"][0]
+
+    swh_storage.metadata_authority_get(authority["type"], authority["url"])
+
+    with pytest.raises(StorageArgumentException, match="Refuse to add"):
+        swh_storage.metadata_authority_add(**authority)
 
     assert mock_memory.call_count == 1
 
@@ -594,37 +575,33 @@ def test_retrying_proxy_storage_origin_metadata_add(swh_storage, sample_data):
 
     """
     ori_meta = sample_data["origin_metadata"][0]
-    origin = ori_meta["origin"]
-    swh_storage.origin_add_one(origin)
-    provider_get = to_provider(ori_meta["provider"])
-    provider_id = swh_storage.metadata_provider_add(**provider_get)
+    swh_storage.origin_add_one({"url": ori_meta["origin_url"]})
+    swh_storage.metadata_authority_add(**sample_data["authority"][0])
+    swh_storage.metadata_fetcher_add(**sample_data["fetcher"][0])
 
-    origin_metadata = swh_storage.origin_metadata_get_by(origin["url"])
+    origin_metadata = swh_storage.origin_metadata_get(
+        ori_meta["origin_url"], ori_meta["authority"]
+    )
     assert not origin_metadata
 
-    swh_storage.origin_metadata_add(
-        origin["url"],
-        ori_meta["discovery_date"],
-        provider_id,
-        ori_meta["tool"],
-        ori_meta["metadata"],
-    )
+    swh_storage.origin_metadata_add(**ori_meta)
 
-    origin_metadata = swh_storage.origin_metadata_get_by(origin["url"])
+    origin_metadata = swh_storage.origin_metadata_get(
+        ori_meta["origin_url"], ori_meta["authority"]
+    )
     assert origin_metadata
 
 
 def test_retrying_proxy_storage_origin_metadata_add_with_retry(
-    swh_storage, sample_data, mocker, fake_hash_collision
+    monkeypatch_sleep, swh_storage, sample_data, mocker, fake_hash_collision
 ):
     """Multiple retries for hash collision and psycopg2 error but finally ok
 
     """
     ori_meta = sample_data["origin_metadata"][0]
-    origin = ori_meta["origin"]
-    swh_storage.origin_add_one(origin)
-    provider_get = to_provider(ori_meta["provider"])
-    provider_id = swh_storage.metadata_provider_add(**provider_get)
+    swh_storage.origin_add_one({"url": ori_meta["origin_url"]})
+    swh_storage.metadata_authority_add(**sample_data["authority"][0])
+    swh_storage.metadata_fetcher_add(**sample_data["fetcher"][0])
     mock_memory = mocker.patch(
         "swh.storage.in_memory.InMemoryStorage.origin_metadata_add"
     )
@@ -633,31 +610,42 @@ def test_retrying_proxy_storage_origin_metadata_add_with_retry(
         # first try goes ko
         fake_hash_collision,
         # second try goes ko
-        psycopg2.IntegrityError("provider_id already inserted"),
+        psycopg2.IntegrityError("foo bar"),
         # ok then!
         None,
     ]
 
-    mock_sleep = mocker.patch(
-        "swh.storage.retry.RetryingProxyStorage" ".origin_metadata_add.retry.sleep"
-    )
-
-    url = origin["url"]
-    ts = ori_meta["discovery_date"]
-    tool_id = ori_meta["tool"]
-    metadata = ori_meta["metadata"]
-
     # No exception raised as insertion finally came through
-    swh_storage.origin_metadata_add(url, ts, provider_id, tool_id, metadata)
+    swh_storage.origin_metadata_add(**ori_meta)
 
     mock_memory.assert_has_calls(
         [  # 3 calls, as long as error raised
-            call(url, ts, provider_id, tool_id, metadata),
-            call(url, ts, provider_id, tool_id, metadata),
-            call(url, ts, provider_id, tool_id, metadata),
+            call(
+                ori_meta["origin_url"],
+                ori_meta["discovery_date"],
+                ori_meta["authority"],
+                ori_meta["fetcher"],
+                ori_meta["format"],
+                ori_meta["metadata"],
+            ),
+            call(
+                ori_meta["origin_url"],
+                ori_meta["discovery_date"],
+                ori_meta["authority"],
+                ori_meta["fetcher"],
+                ori_meta["format"],
+                ori_meta["metadata"],
+            ),
+            call(
+                ori_meta["origin_url"],
+                ori_meta["discovery_date"],
+                ori_meta["authority"],
+                ori_meta["fetcher"],
+                ori_meta["format"],
+                ori_meta["metadata"],
+            ),
         ]
     )
-    assert mock_sleep.call_count == 2
 
 
 def test_retrying_proxy_swh_storage_origin_metadata_add_failure(
@@ -672,17 +660,10 @@ def test_retrying_proxy_swh_storage_origin_metadata_add_failure(
     mock_memory.side_effect = StorageArgumentException("Refuse to add always!")
 
     ori_meta = sample_data["origin_metadata"][0]
-    origin = ori_meta["origin"]
-    swh_storage.origin_add_one(origin)
-
-    url = origin["url"]
-    ts = ori_meta["discovery_date"]
-    provider_id = "provider_id"
-    tool_id = ori_meta["tool"]
-    metadata = ori_meta["metadata"]
+    swh_storage.origin_add_one({"url": ori_meta["origin_url"]})
 
     with pytest.raises(StorageArgumentException, match="Refuse to add"):
-        swh_storage.origin_metadata_add(url, ts, provider_id, tool_id, metadata)
+        swh_storage.origin_metadata_add(**ori_meta)
 
     assert mock_memory.call_count == 1
 
@@ -713,7 +694,7 @@ def test_retrying_proxy_swh_storage_origin_visit_update(swh_storage, sample_data
 
 
 def test_retrying_proxy_swh_storage_origin_visit_update_retry(
-    swh_storage, sample_data, mocker, fake_hash_collision
+    monkeypatch_sleep, swh_storage, sample_data, mocker, fake_hash_collision
 ):
     """Multiple retries for hash collision and psycopg2 error but finally ok
 
@@ -733,10 +714,6 @@ def test_retrying_proxy_swh_storage_origin_visit_update_retry(
         {"origin": origin_url, "visit": 1},
     ]
 
-    mock_sleep = mocker.patch(
-        "swh.storage.retry.RetryingProxyStorage" ".origin_visit_update.retry.sleep"
-    )
-
     visit_id = 1
     swh_storage.origin_visit_update(origin_url, visit_id, status="full")
 
@@ -747,7 +724,6 @@ def test_retrying_proxy_swh_storage_origin_visit_update_retry(
             call(origin_url, visit_id, metadata=None, snapshot=None, status="full"),
         ]
     )
-    assert mock_sleep.call_count == 2
 
 
 def test_retrying_proxy_swh_storage_origin_visit_update_failure(
@@ -788,7 +764,7 @@ def test_retrying_proxy_storage_directory_add(swh_storage, sample_data):
 
 
 def test_retrying_proxy_storage_directory_add_with_retry(
-    swh_storage, sample_data, mocker, fake_hash_collision
+    monkeypatch_sleep, swh_storage, sample_data, mocker, fake_hash_collision
 ):
     """Multiple retries for hash collision and psycopg2 error but finally ok
 
@@ -802,9 +778,6 @@ def test_retrying_proxy_storage_directory_add_with_retry(
         # ok then!
         {"directory:add": 1},
     ]
-    mock_sleep = mocker.patch(
-        "swh.storage.retry.RetryingProxyStorage" ".directory_add.retry.sleep"
-    )
 
     sample_dir = sample_data["directory"][1]
 
@@ -823,7 +796,6 @@ def test_retrying_proxy_storage_directory_add_with_retry(
             call([Directory.from_dict(sample_dir)]),
         ]
     )
-    assert mock_sleep.call_count == 2
 
 
 def test_retrying_proxy_swh_storage_directory_add_failure(
@@ -867,7 +839,7 @@ def test_retrying_proxy_storage_revision_add(swh_storage, sample_data):
 
 
 def test_retrying_proxy_storage_revision_add_with_retry(
-    swh_storage, sample_data, mocker, fake_hash_collision
+    monkeypatch_sleep, swh_storage, sample_data, mocker, fake_hash_collision
 ):
     """Multiple retries for hash collision and psycopg2 error but finally ok
 
@@ -882,10 +854,6 @@ def test_retrying_proxy_storage_revision_add_with_retry(
         {"revision:add": 1},
     ]
 
-    mock_sleep = mocker.patch(
-        "swh.storage.retry.RetryingProxyStorage" ".revision_add.retry.sleep"
-    )
-
     sample_rev = sample_data["revision"][0]
 
     revision = next(swh_storage.revision_get([sample_rev["id"]]))
@@ -903,7 +871,6 @@ def test_retrying_proxy_storage_revision_add_with_retry(
             call([Revision.from_dict(sample_rev)]),
         ]
     )
-    assert mock_sleep.call_count == 2
 
 
 def test_retrying_proxy_swh_storage_revision_add_failure(
@@ -945,7 +912,7 @@ def test_retrying_proxy_storage_release_add(swh_storage, sample_data):
 
 
 def test_retrying_proxy_storage_release_add_with_retry(
-    swh_storage, sample_data, mocker, fake_hash_collision
+    monkeypatch_sleep, swh_storage, sample_data, mocker, fake_hash_collision
 ):
     """Multiple retries for hash collision and psycopg2 error but finally ok
 
@@ -960,10 +927,6 @@ def test_retrying_proxy_storage_release_add_with_retry(
         {"release:add": 1},
     ]
 
-    mock_sleep = mocker.patch(
-        "swh.storage.retry.RetryingProxyStorage" ".release_add.retry.sleep"
-    )
-
     sample_rel = sample_data["release"][0]
 
     release = next(swh_storage.release_get([sample_rel["id"]]))
@@ -981,7 +944,6 @@ def test_retrying_proxy_storage_release_add_with_retry(
             call([Release.from_dict(sample_rel)]),
         ]
     )
-    assert mock_sleep.call_count == 2
 
 
 def test_retrying_proxy_swh_storage_release_add_failure(
@@ -1023,7 +985,7 @@ def test_retrying_proxy_storage_snapshot_add(swh_storage, sample_data):
 
 
 def test_retrying_proxy_storage_snapshot_add_with_retry(
-    swh_storage, sample_data, mocker, fake_hash_collision
+    monkeypatch_sleep, swh_storage, sample_data, mocker, fake_hash_collision
 ):
     """Multiple retries for hash collision and psycopg2 error but finally ok
 
@@ -1038,10 +1000,6 @@ def test_retrying_proxy_storage_snapshot_add_with_retry(
         {"snapshot:add": 1},
     ]
 
-    mock_sleep = mocker.patch(
-        "swh.storage.retry.RetryingProxyStorage" ".snapshot_add.retry.sleep"
-    )
-
     sample_snap = sample_data["snapshot"][0]
 
     snapshot = swh_storage.snapshot_get(sample_snap["id"])
@@ -1059,7 +1017,6 @@ def test_retrying_proxy_storage_snapshot_add_with_retry(
             call([Snapshot.from_dict(sample_snap)]),
         ]
     )
-    assert mock_sleep.call_count == 2
 
 
 def test_retrying_proxy_swh_storage_snapshot_add_failure(
